#include <bits/stdc++.h>

using namespace std;
using ll = long long;

constexpr ll infLL = 3e18;

struct Tag {
    ll add = 0;
    Tag() = default;
    Tag(ll x) : add(x) {}

    void tagapply(const Tag &oth) {
        add += oth.add;
    }
};

constexpr int inf = 1e9 + 7;

struct Info {
    int mn = 1e9, mx = -1e9;
    pair<int, int> minPlus{inf, -1}, maxMinus{-inf, -1};
    Info() = default;
    Info(int x) : mn(x), mx(x) {}

    void infoapply(const Tag &oth) {
        mn += oth.add;
        mx += oth.add;
    }
};

Info operator+(const Info &l, const Info &r) {
    Info res = l;
    res.mn = min(res.mn, l.mn);
    res.mx = max(res.mx, r.mx);
    res.minPlus = min(res.minPlus, r.minPlus);
    res.maxMinus = max(res.maxMinus, r.maxMinus);
    return res;
}

struct LazySegmentTree {
    vector<Info> tree;
    vector<Tag> tag;
    int sz = 1;

    void pull(int x) {
        tree[x] = tree[x << 1] + tree[x << 1 | 1];
    }

    void tapply(int x, const Tag &tg) {
        tree[x].infoapply(tg);
        tag[x].tagapply(tg);
    }

    void push(int x) {
        if (tag[x].add != 0) {
            tapply(x << 1, tag[x]);
            tapply(x << 1 | 1, tag[x]);
            tag[x] = {};
        }
    }

    void init(const vector<Info> &a) {
        int n = a.size();
        sz = 1 << __lg(n) + !!(n & (n - 1));
        tree.assign(sz << 1, {});
        tag.assign(sz << 1, {});
        for (int i = 0; i < n; ++i) {
            tree[i] = a[i];
        }
        for (int i = sz - 1; i > 0; --i) {
            pull(i);
        }
    }

    Info rangeSum(int l, int r, int x, int lx, int rx) {
        if (l >= rx || lx >= r) {
            return {};
        }
        if (l <= lx && rx <= r) {
            return tree[x];
        }
        push(x);
        int mid = lx + rx >> 1;
        return rangeSum(l, r, x << 1, lx, mid) + rangeSum(l, r, x << 1 | 1, mid, rx);
    }

    Info rangeSum(int l, int r) {
        return rangeSum(l, r, 1, 0, sz);
    }

    void modify(int i, const Info &val, int x, int lx, int rx) {
        if (lx + 1 == rx) {
            tree[x] = val;
            return;
        }
        push(x);
        int mid = lx + rx >> 1;
        if (i < mid) {
            modify(i, val, x << 1, lx, mid);
        } else {
            modify(i, val, x << 1 | 1, mid, rx);
        }
        pull(x);
    }

    void modify(int i, const Info &val) {
        modify(i, val, 1, 0, sz);
    }

    void rangeApply(int l, int r, const Tag &tg, int x, int lx, int rx) {
        if (l >= rx || lx >= r) {
            return;
        }
        if (l <= lx && rx <= r) {
            return tapply(x, tg);
        }
        int mid = lx + rx >> 1;
        push(x);
        rangeApply(l, r, tg, x << 1, lx, mid);
        rangeApply(l, r, tg, x << 1 | 1, mid, rx);
        pull(x);
    }

    void rangeApply(int l, int r, const Tag &tg) {
        rangeApply(l, r, tg, 1, 0, sz);
    }

    int findLast(int l, int r, auto &&pred, int x, int lx, int rx) { // last i \in [l, r] : pred(i) = true
        if (l >= rx || lx >= r || pred(tree[x])) {
            return -1;
        }
        if (lx + 1 == rx) {
            return lx;
        }
        int mid = lx + rx >> 1;
        push(x);
        int res = findLast(l, r, pred, x << 1 | 1, mid, rx);
        if (res == -1) {
            res = findLast(l, r, pred, x << 1, lx, mid);
        }
        return res;
    }

    int findLast(int l, int r, auto &&pred) {
        return findLast(l, r, pred, 1, 0, sz);
    }
};

void solve() {
    int n, m;
    cin >> n >> m;
    vector<int> X(m);
    for (int i = 0; i < m; ++i) {
        cin >> X[i];
    }
    reverse(X.begin(), X.end());
    LazySegmentTree tree;
    tree.init(vector(m, Info{}));
    ll ans = 0;
    vector<int> bracket(m);
    for (int i = 0; i < m; ++i) {
        if (X[i] >= 0) {
            bracket[i] = 1;
        } else {
            bracket[i] = -1;
        }
        int bal = (i > 0 ? tree.rangeSum(i - 1, i).mx : 0) + bracket[i];
        ans += bracket[i] * X[i];
        Info cur(bal);
        if (bracket[i] == 1) {
            cur.minPlus = pair(X[i], i);
        } else {
            cur.maxMinus = pair(X[i], i);
        }
        tree.modify(i, cur);
        if (bal > n) {
            pair<ll, int> newBest{-infLL, 0};
            int lastj = tree.findLast(0, i + 1, [&](auto info) {
                return info.mn > 1;
            });
            int lx = lastj + 1;
//            for (int j = i; j >= 0; --j) {
//                if (bal[j] <= 1) {
//                    break;
//                }
//                if (bracket[j] == 1) {
//                    ll cur = ans - 2 * X[j];
//                    newBest = max(newBest, pair(cur, j));
//                }
//            }
            auto [minx, optj] = tree.rangeSum(lx, i + 1).minPlus;
            ans = ans - 2 * minx;
            bracket[optj] *= -1;
            Info cur = tree.rangeSum(optj, optj + 1);
            cur.maxMinus = cur.minPlus;
            cur.minPlus = {inf, -1};
            tree.modify(optj, cur);
            tree.rangeApply(optj, i + 1, Tag(-2));
//            for (int k = j; k <= i; ++k) {
//                bal[k] -= 2;
//            }
        } else if (bal < 0) {
            pair<ll, int> newBest{-infLL, 0};
            int lastj = tree.findLast(0, i + 1, [&](auto info) {
                return info.mx < n - 1;
            });
            int lx = lastj + 1;
//            for (int j = i; j >= 0; --j) {
//                if (bal[j] >= n - 1) {
//                    break;
//                }
//                if (bracket[j] == -1) {
//                    ll cur = ans + 2 * X[j];
//                    newBest = max(newBest, pair(cur, j));
//                }
//            }
            auto [maxx, optj] = tree.rangeSum(lx, i + 1).maxMinus;
            ans = ans + 2 * maxx;
            bracket[optj] *= -1;
            Info cur = tree.rangeSum(optj, optj + 1);
            cur.minPlus = cur.maxMinus;
            cur.maxMinus = {-inf, -1};
            tree.modify(optj, cur);
            tree.rangeApply(optj, i + 1, Tag(2));
//            for (int k = j; k <= i; ++k) {
//                bal[k] += 2;
//            }
        }
//        for (int j = 0; j <= i; ++j) {
//            cout << tree.rangeSum(j, j + 1).mx << " ";
//        }
//        cout << endl;
    }
    cout << ans << '\n';
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int test = 1;
    cin >> test;
    while (test--) {
        solve();
    }

    return 0;
}