#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <map>
#include <unordered_set>
#include <cmath>

using namespace std;
using ll = long long;

constexpr int INF = 1e9 + 7;
constexpr ll LINF = 1e18 + 7;

struct SegmentTree {
    struct node {
        ll s = 0;
        ll p = 0;
    };
    vector<node>tree;

    inline int L(int v) { return 2 * v + 1; }
    inline int R(int v) { return 2 * v + 2; }

    void push(int v, int l, int r) {
        int mid = (l + r) / 2;
        tree[L(v)].s += tree[v].p * (mid - l);
        tree[L(v)].p += tree[v].p;
        tree[R(v)].s += tree[v].p * (r - mid);
        tree[R(v)].p += tree[v].p;
    }

    void add(int ql, int qr, ll x, int l, int r, int v) {
        if (l >= qr || r <= ql) return;
        if (ql <= l && r <= qr) {
            tree[v].s += x * (r - l);
            tree[v].p += x;
            return;
        }
        int mid = (l + r) / 2;
        push(v, l, r);
        add(ql, qr, x, l, mid, L(v));
        add(ql, qr, x, mid, r, R(v));
        tree[v].s = tree[L(v)].s + tree[R(v)].s;
    }

    void build(vector<int>& a, int l, int r, int v) {
        if (r - l == 1) {
            tree[v].s = a[l];
            return;
        }
        int mid = (l + r) / 2;
        build(a, l, mid, L(v));
        build(a, mid, r, R(v));
        tree[v].s = tree[L(v)].s + tree[R(v)].s;
    }

    SegmentTree(vector<int>& a) {
        int n = a.size();
        tree.resize(4 * n);
        build(a, 0, n, 0);
    }
};

void solve() {
    int n, m;
    cin >> n >> m;
    vector<ll>x(m);
    for (int i = 0; i < m; ++i) {
        cin >> x[i];
    }
    multiset<ll>s;
    for (int i = 0; i < n; ++i) s.insert(0);
    for (int i = 0; i < m; ++i) {
        ll a = *s.begin();
        s.erase(s.begin());
        s.insert(x[i] - a);
    }
    ll res = 0;
    for (auto x : s) res += x;
    cout << res << "\n";
}

int main()
{
    cin.tie(0)->sync_with_stdio(0);
    int t;
    cin >> t;
    while (t--) solve();

}
