#include <iostream>
#include <vector>
#include <set>
#include <algorithm>

using namespace std;

const long long INF = 1e17;

struct tree{
    vector<int> tr;
    vector<int> pr;
    int n;

    tree(int n) : n(n) {
        tr.resize(n * 4, 0);
        pr.resize(n * 4, 0);
    }

    void push(int id, int l, int r) {
        tr[id] += pr[id];
        if (r - l != 1) {
            pr[id * 2 + 1] += pr[id];
            pr[id * 2 + 2] += pr[id];
        }
        pr[id] = 0;
    }

    void add(int id, int l, int r, int ql, int qr, int x) {
        push(id, l, r);
        if (ql <= l && r <= qr) {
            pr[id] += x;
            push(id, l, r);
            return;
        }
        if (r <= ql || qr <= l) {
            return;
        }
        int mid = (l + r) / 2;
        add(id * 2 + 1, l, mid, ql, qr, x);
        add(id * 2 + 2, mid, r, ql, qr, x);
        tr[id] = min(tr[id * 2 + 1], tr[id * 2 + 2]);
    }

    int ask(int id, int l, int r, int ql, int qr) {
        push(id, l, r);
        if (ql <= l && r <= qr) {
            return tr[id];
        }
        if (r <= ql || qr <= l) {
            return 1e9;
        }
        int mid = (l + r) / 2;
        return min(ask(id * 2 + 1, l, mid, ql, qr), ask(id * 2 + 2, mid, r, ql, qr));
    }
};

void add(tree &tr, int id, int t) {
    tr.add(0, 0, tr.n, id, tr.n, t);
}

int getmin(tree &tr) {
    return tr.ask(0, 0, tr.n, 0, tr.n);
}

int get(tree &tr, int id) {
    return tr.ask(0, 0, tr.n, id, id + 1);
}

void solve() {
    int n, m;
    cin >> m >> n;
    vector<long long> v(n);
    for (int i = 0; i < n; i++) {
        cin >> v[i];
    }
    reverse(v.begin(), v.end());
    tree tr(n);
    add(tr, 0, 1);
    set<pair<long long, int>> st;
    vector<int> sgn(n);
    sgn[0] = 1;
    for (int i = 1; i < n; i++) {
        long long x = v[i];
        st.insert({v[i], i});
        add(tr, i, 1);
        sgn[i] = 1;
        if (get(tr, i) > m) {  
            while (true) {
                auto [x, id] = *(st.begin());
                add(tr, id, -2);
                sgn[id] = -1;
                st.erase(st.begin());
                if (getmin(tr) >= 0) {
                    break;
                } else {
                    add(tr, id, 2);
                    sgn[id] = 1;
                }
            }
        }
    }
    long long ans = 0;
    for (int i = 0; i < n; i++) {
        if (sgn[i] == 1) {
            ans += v[i];
        } else {
            ans -= v[i];
        }
    }
    cout << ans << '\n';
}

int main()
{
    cin.tie(0); cout.tie(0); ios_base::sync_with_stdio(false);
    int t;
    cin >> t;
    for (int i = 0; i < t; i++) {
        solve();
    }

    return 0;
}