// tch1cherin GM!
#pragma GCC optimize("Ofast,unroll-loops")
#include <bits/stdc++.h>
#pragma GCC target("avx2,tune=native")
using namespace std;

struct segment_tree {
  struct node {
    int max_value, tag;
  };

  node merge(node a, node b) {
    return {max(a.max_value, b.max_value), 0};
  }

  int size;
  vector<node> tree;

  segment_tree() = default;

  segment_tree(int n) {
    size = 1;
    while (size < n) {
      size *= 2;
    }
    tree.assign(size * 2, {0, 0});
  }

  void apply(int x, int value) {
    tree[x].max_value += value;
    tree[x].tag += value;
  }

  void propagate(int x) {
    if (tree[x].tag != 0) {
      apply(x * 2, tree[x].tag);
      apply(x * 2 + 1, tree[x].tag);
      tree[x].tag = 0;
    }
  }

  void update(int l, int r, int value, int x, int lx, int rx) {
    if (lx >= r || rx <= l) {
      return;
    } else if (lx >= l && rx <= r) {
      apply(x, value);
    } else {
      propagate(x);
      int mid = (lx + rx) / 2;
      update(l, r, value, x * 2, lx, mid);
      update(l, r, value, x * 2 + 1, mid, rx);
      tree[x] = merge(tree[x * 2], tree[x * 2 + 1]);
    }
  }

  void update(int l, int r, int value) {
    update(l, r, value, 1, 0, size);
  }

  int query(int l, int r) {
    if (l >= r) {
      return 0;
    }
    return query(l, r, 1, 0, size);
  }

  int query(int l, int r, int x, int lx, int rx) {
    if (lx >= r || rx <= l) {
      return 0;
    } else if (lx >= l && rx <= r) {
      return tree[x].max_value;
    } else {
      propagate(x);
      int mid = (lx + rx) / 2;
      int left = query(l, r, x * 2, lx, mid);
      int right = query(l, r, x * 2 + 1, mid, rx);
      return max(left, right);
    }
  }
};

constexpr int MAX_N = 300000;

segment_tree ST;
int S[MAX_N];
vector<int> graph[MAX_N];
int tin[MAX_N], tout[MAX_N], sz[MAX_N];
int head[MAX_N], end_h[MAX_N], par[MAX_N];
int T = 0;

void DFS(int u) {
  sz[u] = 1;
  for (int &v : graph[u]) {
    DFS(v);
    sz[u] += sz[v];
    if (sz[v] > sz[graph[u][0]]) {
      swap(v, graph[u][0]);
    }
  }
}

void HLD(int u) {
  tin[u] = T++;
  for (int v : graph[u]) {
    if (v == graph[u][0]) {
      head[v] = head[u];
    } else {
      head[v] = v;
    }
    HLD(v);
  }
  tout[u] = T;
}

void hld_update(int u) {
  while (u >= 0) {
    ST.update(tin[head[u]], tin[u] + 1, +1);
    u = par[head[u]];
  }
}

int hld_query(int u) {
  int answer = ST.query(tin[u] + 1, ST.size);
  while (u != -1) {
    if (par[head[u]] != -1) {
      answer = max(answer, ST.query(tin[par[head[u]]] + 1, tin[head[u]]));
    }
    u = par[head[u]];
  }
  return answer;
}

int main() {
  cin.tie(nullptr)->sync_with_stdio(false);
  int n, q;
  cin >> n >> q;
  ST = segment_tree(n);
  par[0] = -1;
  for (int i = 1; i < n; i++) {
    cin >> par[i];
    graph[--par[i]].push_back(i);
  }
  for (int i = 0; i < n; i++) {
    cin >> S[i];
  }
  DFS(0);
  head[0] = 0;
  HLD(0);
  priority_queue<pair<int, int>> Q;
  for (int v : graph[0]) {
    Q.push({S[v], v});
  }
  vector<int> answer(n);
  for (int it = 1; it < n; it++) {
    auto [val, u] = Q.top();
    Q.pop();
    for (int v : graph[u]) {
      Q.push({S[v], v});
    }
    answer[u] = it - hld_query(u);
    hld_update(u);
  }
  for (int i = 0; i < q; i++) {
    int v;
    cin >> v;
    cout << answer[--v] << " \n"[i + 1 == q];
  }
}
