#include <iostream>
#include <vector>
#include <stack>
#include <deque>
#include <queue>
#include <set>
#include <bitset>
#include <unordered_set>
#include <map>
#include <unordered_map>
#include <cmath>
#include <iomanip>
#include <algorithm>
#include <random>

using namespace std;

random_device rd;
mt19937 rnd(rd());

// #define int long long
using pii = pair<int, int>;

const int max_n = 3e5 + 5;

struct node {
    int mx, id, add;
    explicit node(int mx_ = 0, int add_ = 0, int id_ = -1) {
        mx = mx_, id = id_, add = add_;
    }
};

struct segment_tree {
    vector<node> tree;
    vector<int> a;
    int n;
    segment_tree(vector<int> a_) {
        a = a_;
        n = a.size();
        tree.resize(4 * n + 5);
    }
    node merge(node l, node r) {
        node res;
        res.mx = max(l.mx, r.mx);
        if (res.mx == l.mx) {
            res.id = l.id;
        } else {
            res.id = r.id;
        }
        return res;
    }
    void build(int v, int l, int r) {
        if (r - l == 1) {
            tree[v].mx = a[l];
            tree[v].id = l;
            return;
        }
        int m = (l + r) / 2;
        build(2 * v + 1, l, m);
        build(2 * v + 2, m, r);
        tree[v] = merge(tree[2 * v + 1], tree[2 * v + 2]);
    }
    void apply(int v, int x) {
        tree[v].mx += x;
        tree[v].add += x;
    }
    void push(int v) {
        apply(2 * v + 1, tree[v].add);
        apply(2 * v + 2, tree[v].add);
        tree[v].add = 0;
    }
    node t_get(int v, int l, int r, int ql, int qr) {
        if (ql <= l && r <= qr) {
            return tree[v];
        }
        if (qr <= l || r <= ql) {
            return node();
        }
        push(v);
        int m = (l + r) / 2;
        return merge(t_get(2 * v + 1, l, m, ql, qr), t_get(2 * v + 2, m, r, ql, qr));
    }
    node get(int l, int r) {
        return t_get(0, 0, n, l, r);
    }
    void t_upd(int v, int l, int r, int ql, int qr, int x) {
        if (ql <= l && r <= qr) {
            apply(v, x);
            return;
        }
        if (qr <= l || r <= ql) {
            return;
        }
        int m = (l + r) / 2;
        push(v);
        t_upd(2 * v + 1, l, m, ql, qr, x);
        t_upd(2 * v + 2, m, r, ql, qr, x);
        tree[v] = merge(tree[2 * v + 1], tree[2 * v + 2]);
    }
    void upd(int l, int r, int x) {
        t_upd(0, 0, n, l, r, x);
    }
};

vector<int> g[max_n];

vector<int> tin(max_n), tout(max_n);
vector<int> sz(max_n);
vector<int> way_b(max_n);
int tr = 0;

void hld_reorder(int v) {
    sz[v] = 1;
    for (int u : g[v]) {
        hld_reorder(u);
        sz[v] += sz[u];
    }
    for (int i = 0; i < g[v].size(); i++) {
        if (sz[g[v][i]] > sz[g[v][0]]) {
            swap(g[v][i], g[v][0]);
        }
    }
}

void dfs(int v) {
    tin[v] = tr++;
    if (!g[v].empty()) {
        way_b[g[v][0]] = way_b[v];
        dfs(g[v][0]);
        for (int i = 1; i < g[v].size(); i++) {
            way_b[g[v][i]] = g[v][i];
            dfs(g[v][i]);
        }
    }
    tout[v] = tr;
}

bool is_v_anc_of_u(int v, int u) {
    return tin[v] <= tin[u] && tout[u] <= tout[v];
}

void solve() {
    int n, q; cin >> n >> q;
    vector<int> p(n);
    p[0] = -1;
    for (int i = 1; i < n; i++) {
        cin >> p[i];
        p[i]--;
        g[p[i]].push_back(i);
    }
    hld_reorder(0);
    way_b[0] = 0;
    dfs(0);
    vector<int> a(n);
    for (int i = 0; i < n; i++) {
        cin >> a[i];
    }
    set<pii> s;
    s.insert({-a[0], 0});
    vector<int> ord;
    while (!s.empty()) {
        auto [val, v] = *s.begin();
        s.erase({val, v});
        ord.push_back(v);
        for (int u : g[v]) {
            s.insert({-a[u], u});
        }
    }
    segment_tree tree(vector<int> (n, 0));
    tree.build(0, 0, n);
    vector<int> v_res(n, 0);
  //  for (int i = 0; i < n; i++) {
//        cout << tin[i] << " ";
//    }
//    cout << endl;
    for (int i = 0; i < n; i++) {
        int v = ord[i];
        vector<pii> segs;
        while (v != -1) {
            segs.emplace_back(tin[way_b[v]], tin[v] + 1);
            v = p[way_b[v]];
        }
        reverse(segs.begin(), segs.end());
        v = ord[i];
        int temp = 0;
        temp = max(temp, tree.get(0, segs[0].first).mx);
        temp = max(temp, tree.get(segs.back().second, n).mx);
        for (int i = 0; i + 1 < segs.size(); i++) {
            temp = max(temp, tree.get(segs[i].second, segs[i + 1].first).mx);
        }
        v_res[v] = i - temp;
        for (auto [l, r] : segs) {
          //  cout << "v seg " << v + 1 << " " << l << " " << r << endl;
            tree.upd(l, r, 1);
        }
    }
    while (q--) {
        int v; cin >> v;
        v--;
        cout << v_res[v] << " ";
    }
}

/*
5 4
1 2 2 1
3 5 1 2 4
5 3 1 4
 */

int32_t main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    solve();
}
