#include <bits/stdc++.h>

using namespace std;
using ll = long long;

struct Tag {
    ll add = 0;
    Tag() = default;
    Tag(ll x) : add(x) {}

    void tagapply(const Tag &oth) {
        add += oth.add;
    }
};

struct Info {
    int mx = 0;
    Info() = default;
    Info(int x) : mx(x) {}

    void infoapply(const Tag &oth) {
        mx += oth.add;
    }
};

Info operator+(const Info &l, const Info &r) {
    return max(l.mx, r.mx);
}

struct LazySegmentTree {
    vector<Info> tree;
    vector<Tag> tag;
    int sz = 1;

    void pull(int x) {
        tree[x] = tree[x << 1] + tree[x << 1 | 1];
    }

    void tapply(int x, const Tag &tg) {
        tree[x].infoapply(tg);
        tag[x].tagapply(tg);
    }

    void push(int x) {
        if (tag[x].add != 0) {
            tapply(x << 1, tag[x]);
            tapply(x << 1 | 1, tag[x]);
            tag[x] = {};
        }
    }

    void init(const vector<Info> &a) {
        int n = a.size();
        sz = 1 << __lg(n) + !!(n & (n - 1));
        tree.assign(sz << 1, {});
        tag.assign(sz << 1, {});
        for (int i = 0; i < n; ++i) {
            tree[i] = a[i];
        }
        for (int i = sz - 1; i > 0; --i) {
            pull(i);
        }
    }

    Info rangeSum(int l, int r, int x, int lx, int rx) {
        if (l >= rx || lx >= r) {
            return {};
        }
        if (l <= lx && rx <= r) {
            return tree[x];
        }
        push(x);
        int mid = lx + rx >> 1;
        return rangeSum(l, r, x << 1, lx, mid) + rangeSum(l, r, x << 1 | 1, mid, rx);
    }

    Info rangeSum(int l, int r) {
        return rangeSum(l, r, 1, 0, sz);
    }

    void modify(int i, const Info &val, int x, int lx, int rx) {
        if (lx + 1 == rx) {
            tree[x] = val;
            return;
        }
        push(x);
        int mid = lx + rx >> 1;
        if (i < mid) {
            modify(i, val, x << 1, lx, mid);
        } else {
            modify(i, val, x << 1 | 1, mid, rx);
        }
        pull(x);
    }

    void modify(int i, const Info &val) {
        modify(i, val, 1, 0, sz);
    }

    void rangeApply(int l, int r, const Tag &tg, int x, int lx, int rx) {
        if (l >= rx || lx >= r) {
            return;
        }
        if (l <= lx && rx <= r) {
            return tapply(x, tg);
        }
        int mid = lx + rx >> 1;
        push(x);
        rangeApply(l, r, tg, x << 1, lx, mid);
        rangeApply(l, r, tg, x << 1 | 1, mid, rx);
        pull(x);
    }

    void rangeApply(int l, int r, const Tag &tg) {
        rangeApply(l, r, tg, 1, 0, sz);
    }
};

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int n, q;
    cin >> n >> q;

    vector<int> fa(n);
    vector<vector<int>> adj(n);
    for (int i = 1; i < n; ++i) {
        cin >> fa[i];
        adj[--fa[i]].push_back(i);
    }
    vector<int> s(n);
    for (int i = 0; i < n; ++i) {
        cin >> s[i];
    }

    vector<int> siz(n, 1);
    vector<int> head(n, -1), tin(n), tout(n), ord(n);
    int timer = 0;
    auto dfs1 = [&](auto self, int v) -> void {
        siz[v] = 1;
        for (int &to : adj[v]) {
            self(self, to);
            siz[v] += siz[to];
            if (siz[to] > siz[adj[v][0]]) {
                swap(to, adj[v][0]);
            }
        }
   };
    dfs1(dfs1, 0);
    auto dfs2 = [&](auto self, int v) -> void {
        ord[tin[v] = timer++] = v;
        head[v] = (head[v] == -1 ? v : head[v]);
        for (int to : adj[v]) {
            if (to == adj[v][0]) {
                head[to] = head[v];
            }
            self(self, to);
        }
        tout[v] = timer;
    };
    dfs2(dfs2, 0);

    LazySegmentTree tree;
    tree.init(vector(n, Info{}));

    auto addParents = [&](int v) {
        while (true) {
            int p = head[v];
            tree.rangeApply(tin[p], tin[v] + 1, Tag(1));
            if (p == 0) {
                break;
            }
            v = fa[p];
        }
    };

    auto query = [&](int v) -> int {
        Info res{};
        vector<pair<int, int>> tins;
        while (true) {
            int p = head[v];
            tins.push_back({tin[p], tin[v] + 1});
            if (p == 0) {
                break;
            }
            v = fa[p];
        }
        reverse(tins.begin(), tins.end());
        tins.push_back({n, n});
        tins.insert(tins.begin(), pair{0, 0});
        assert(is_sorted(tins.begin(), tins.end()));
        for (int i = 0; i + 1 < tins.size(); ++i) {
            if (tins[i].second < tins[i + 1].first) {
                res = res + tree.rangeSum(tins[i].second, tins[i + 1].first);
            }
        }
        return res.mx;
    };

    vector<int> answers(n, 1e9);

    set<pair<int, int>, greater<>> t;
    t.emplace(s[0], 0);
    int pos = 0;
    while (!t.empty()) {
        auto [prior, v] = *t.begin();
        t.erase(t.begin());
        pos += 1;
        addParents(v);
//        cout << "process: " << v << endl;
//        for (int i = 0; i < n; ++i) {
//            cout << ord[i] << ": " << tree.rangeSum(i, i + 1).mx << endl;
//        }
        int mx = query(v);
        answers[v] = pos - mx - 1;
        for (int to : adj[v]) {
            t.emplace(s[to], to);
        }
    }

    while (q--) {
        int v;
        cin >> v;
        cout << answers[v - 1] << " ";
    }

    return 0;
}