#include <iostream>
#include <vector>
#include <string>
#include <numeric>
#include <algorithm>
#include <unordered_map>
#include <map>
#include <set>
#include <unordered_set>

using namespace std;

using ll = long long;

struct segmentTree {
	vector<ll> tree;
	vector<ll> hmass;

	segmentTree(ll n) {
		tree.resize(n * 4);
		hmass.resize(n * 4);
	}

	void push(ll i, ll l, ll r) {
		if (r - l > 1) {
			tree[i * 2 + 1] += hmass[i];
			tree[i * 2 + 2] += hmass[i];
			hmass[i * 2 + 1] += hmass[i];
			hmass[i * 2 + 2] += hmass[i];
		}
		hmass[i] = 0;
	}

	ll get(ll i, ll l, ll r, ll ql, ll qr) {
		if (l >= ql && r <= qr) return tree[i];
		if (l >= qr || r <= ql) return 0;
		push(i, l, r);
		ll m = (l + r) / 2;
		return max(get(i * 2 + 1, l, m, ql, qr), get(i * 2 + 2, m, r, ql, qr));
	}

	void increment(ll i, ll l, ll r, ll ql, ll qr) {
		if (l >= ql && r <= qr) {
			tree[i]++;
			hmass[i]++;
			return;
		}
		if (l >= qr || r <= ql) return;
		push(i, l, r);
		ll m = (l + r) / 2;
		increment(i * 2 + 1, l, m, ql, qr);
		increment(i * 2 + 2, m, r, ql, qr);
		tree[i] = max(tree[i * 2 + 1], tree[i * 2 + 2]);
	}
};

vector<vector<ll>> graph;
vector<ll> competence;

vector<ll> up;
vector<ll> euler;
vector<ll> eulerPos;

void buildHld(ll i) {
	euler.push_back(i);
	eulerPos[i] = euler.size() - 1;
	if (!graph[i].empty()) up[graph[i][0]] = up[i];
	for (auto p : graph[i]) buildHld(p);
}

int main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	ll n, q; cin >> n >> q;
	vector<ll> parent(n, -1);
	graph.resize(n);
	for (int i = 1; i < n; i++) {
		ll p; cin >> p; p--;
		graph[p].push_back(i);
		parent[i] = p;
	}
	vector<int> sz(n, 1);
	for (int i = n - 1; i >= 0; i--) {
		for (auto p : graph[i]) sz[i] += sz[p];
		int largest = 0;
		for (int j = 0; j < graph[i].size(); j++) {
			if (sz[graph[i][j]] > sz[graph[i][largest]]) largest = j;
		}
		if (!graph[i].empty()) swap(graph[i][0], graph[i][largest]);
	}
	competence.resize(n);
	for (int i = 0; i < n; i++) cin >> competence[i];

	eulerPos.resize(n);
	up.resize(n);
	iota(up.begin(), up.end(), 0);
	buildHld(0);
	segmentTree st(n);

	vector<ll> ans(n, -1);
	set<pair<ll, ll>> kid;
	kid.insert({ competence[0], 0 });
	ll num = 0;
	while (!kid.empty()) {
		auto next = *kid.rbegin();
		kid.erase(next);

		ll maxBranch = 0;
		// get maxBranch somehow
		vector<pair<ll, ll>> segments;
		ll i = next.second;
		while (i != -1) {
			ll u = up[i];
			ll l = eulerPos[u], r = eulerPos[i];
			if (l > r) swap(l, r);
			r++;
			segments.push_back({ l, r });
			i = parent[u];
		}
		sort(segments.begin(), segments.end());
		for (auto s : segments) {
			st.increment(0, 0, n, s.first, s.second);
		}
		vector<pair<ll, ll>> segments2;
		for (int i = 0; i < segments.size() - 1; i++) {
			if (segments[i + 1].first - segments[i].second > 0) segments2.push_back({ segments[i].second, segments[i + 1].first });
		}
		if (segments[0].first > 0) segments2.push_back({ 0, segments[0].first });
		if (segments.back().second < n) segments2.push_back({ segments.back().second, n });
		for (auto s : segments2) {
			maxBranch = max(maxBranch, st.get(0, 0, n, s.first, s.second));
		}
		// end getting maxbranch
  		ans[next.second] = num -maxBranch;
		
		for (auto p : graph[next.second]) {
			kid.insert({ competence[p], p });
		}
		num++;
	}

	while (q--) {
		ll v; cin >> v; v--;
		cout << ans[v] << endl;
	}
}