#include <iostream>
#include <vector>
#include <numeric>
#include <algorithm>
#include <cmath>
#include <set>
#include <random>
using namespace std;

#define all(x) x.begin(), x.end()
#define sz(x) static_cast<lng>(x.size())

using lng = long long;

mt19937 rnd(324);

struct Node {
	int value, priority, subtree_sz;
	lng subtree_sum;
	Node* lv, * rv;
	Node(int x = 0) : value(x), priority(rnd()), subtree_sz(1), lv(nullptr), rv(nullptr), subtree_sum(x) {}
	void update() {
		subtree_sz = 1;
		subtree_sum = value;
		if (lv) {
			subtree_sz += lv->subtree_sz;
			subtree_sum += lv->subtree_sum;
		}
		if (rv) {
			subtree_sz += rv->subtree_sz;
			subtree_sum += rv->subtree_sum;
		}
	}
};

Node* merge(Node* left, Node* right) {
	if (!left) {
		return right;
	}
	if (!right) {
		return left;
	}
	if (left->priority < right->priority) {
		left->rv = merge(left->rv, right);
		left->update();
		return left;
	}
	else {
		right->lv = merge(left, right->lv);
		right->update();
		return right;
	}
}

pair<Node*, Node*> split_by_val(Node* cur, int X) {
	if (!cur) {
		return { nullptr, nullptr };
	}
	if (cur->value <= X) {
		auto p = split_by_val(cur->rv, X);
		cur->rv = p.first;
		cur->update();
		return { cur, p.second };
	}
	else {
		auto p = split_by_val(cur->lv, X);
		cur->lv = p.second;
		cur->update();
		return { p.first, cur };
	}
}

pair<Node*, Node*> split_by_sz(Node* cur, int K) {
	if (!cur) {
		return { nullptr, nullptr };
	}
	int left_size = (cur->lv ? cur->lv->subtree_sz : 0) + 1;
	if (left_size <= K) {
		auto p = split_by_sz(cur->rv, K - left_size);
		cur->rv = p.first;
		cur->update();
		return { cur, p.second };
	}
	else {
		auto p = split_by_sz(cur->lv, K);
		cur->lv = p.second;
		cur->update();
		return { p.first, cur };
	}
}

Node* root = nullptr;

void add(int X) {
	auto p = split_by_val(root, X);
	root = merge(merge(p.first, new Node(X)), p.second);
}

void erase(int X) {
	auto p = split_by_val(root, X - 1);
	auto q = split_by_sz(p.second, 1);
	root = merge(p.first, q.second);
}

lng get_k_max(int K) {
	int sz = (root ? root->subtree_sz : 0);
	auto p = split_by_sz(root, sz - K);
	lng res = (p.second ? p.second->subtree_sum : 0ll);
	root = merge(p.first, p.second);
	return res;
}

void solve() {
	int k, n; cin >> n >> k;
	vector<lng> a(n);
	for (lng& i : a) {
		cin >> i;
	}
	for (int i = 0; i < n; i++) {
		add(a[i] + 1);
	}
	lng ans = 0;
	for (int i = 0; i < n; i++) {
		erase(a[i] + 1);
		ans = max(ans, get_k_max(k - 1) + a[i] + i + 1ll);
		add(a[i]);
	}
	cout << ans << '\n';
}

int main() { 
	cin.tie(nullptr)->sync_with_stdio(false);
	int t = 1;
	while (t--) {
		solve();
	}
	return 0;
}