#include <iostream>
#include <vector>
#include <string>
#include <numeric>
#include <algorithm>
#include <unordered_map>
#include <map>
#include <set>
#include <unordered_set>

using namespace std;

using ll = long long;

struct node {
	ll minId = 0;
	ll sum = 0;
};

node merge(node a, node b, vector<ll>& vec) {
	if (a.sum == -1) return b;
	if (b.sum == -1) return a;
	node n;
	n.sum = a.sum + b.sum;
	n.minId = a.minId;
	if (vec[b.minId] < vec[a.minId]) n.minId = b.minId;
	return n;
}

struct SegmentTree {
	vector<ll> vec;
	vector<node> segTree;

	void build(int i, int l, int r) {
		node n;
		n.minId = l;
		segTree[i] = n;
		if (r - l > 1) {
			int m = (l + r) / 2;
			build(i * 2 + 1, l, m);
			build(i * 2 + 2, m, r);
		}
	}

	SegmentTree(ll n) {
		vec.resize(n);
		segTree.resize(n * 4);
		build(0, 0, n);
	}

	node get(ll i, ll l, ll r, ll ql, ll qr) {
		if (l >= ql && r <= qr) return segTree[i];
		if (l >= qr || r <= ql) {
			node n;
			n.sum = -1;
			return n;
		}
		ll m = (l + r) / 2;
		return merge(get(i * 2 + 1, l, m, ql, qr), get(i * 2 + 2, m, r, ql, qr), vec);
	}

	void set(ll i, ll l, ll r, ll id, ll x) {
		if (r - l == 1) {
			vec[l] = x;
			segTree[i].sum = x;
			return;
		}
		ll m = (l + r) / 2;
		if (id < m) set(i * 2 + 1, l, m, id, x);
		else set(i * 2 + 2, m, r, id, x);
		segTree[i] = merge(segTree[i * 2 + 1], segTree[i * 2 + 2], vec);
	}
};

int main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	ll n, k; cin >> n >> k;
	vector<ll> vec(n);
	for (int i = 0; i < n; i++) cin >> vec[i];
	if (k == 1) {
		ll ans = 0;
		for (int i = 0; i < n; i++) {
			ans = max(ans, vec[i] + i + 1);
		}
		cout << ans << endl;
		return 0;
	}
	ll ans = 0;
	SegmentTree st(k - 1);
	vector<ll> vv(k - 1);
	for (int i = 0; i < n; i++) {
		node mn = st.get(0, 0, k - 1, 0, k - 1);
		ll lans = vec[i] + i + 1 + mn.sum;
		ans = max(ans, lans);
		if (vv[mn.minId] < vec[i]) {
			vv[mn.minId] = vec[i];
			st.set(0, 0, k - 1, mn.minId, vec[i]);
		}
	}
	cout << ans << endl;
}