#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <map>
#include <unordered_set>
#include <tuple>
#include <queue>
#include <cmath>

using namespace std;
using ll = long long;

constexpr int INF = 1e9 + 7;
constexpr ll LINF = 1e18 + 7;

struct SegmentTree {
    struct node {
        ll mx = 0;
        ll p = 0;
    };
    vector<node>tree;

    inline int L(int v) { return 2 * v + 1; }
    inline int R(int v) { return 2 * v + 2; }


    void push(int v, int l, int r) {
        int mid = (l + r) / 2;
        tree[L(v)].mx += tree[v].p;
        tree[L(v)].p += tree[v].p;
        tree[R(v)].mx += tree[v].p;
        tree[R(v)].p += tree[v].p;
        tree[v].p = 0;
    }

    void add(int ql, int qr, ll x, int l, int r, int v) {
        if (l >= qr || r <= ql) return;
        if (ql <= l && r <= qr) {
            tree[v].mx += x;
            tree[v].p += x;
            return;
        }
        int mid = (l + r) / 2;
        push(v, l, r);
        add(ql, qr, x, l, mid, L(v));
        add(ql, qr, x, mid, r, R(v));
        tree[v].mx = max(tree[L(v)].mx, tree[R(v)].mx);
    }

    int getmax(int ql, int qr, int l, int r, int v) {
        if (l >= qr || r <= ql) return 0;
        if (ql <= l && r <= qr) return tree[v].mx;
        int mid = (l + r) / 2;
        push(v, l, r);
        return max(getmax(ql, qr, l, mid, L(v)), getmax(ql, qr, mid, r, R(v)));
    }

    void build(vector<int>& a, int l, int r, int v) {
        if (r - l == 1) {
            tree[v].mx = a[l];
            return;
        }
        int mid = (l + r) / 2;
        build(a, l, mid, L(v));
        build(a, mid, r, R(v));
        tree[v].mx = max(tree[L(v)].mx, tree[R(v)].mx);
    }

    SegmentTree() {
        tree = {};
    }
    SegmentTree(int n) {
        tree.resize(4 * n);
    }
    SegmentTree(vector<int>a) {
        int n = a.size();
        tree.resize(4 * n);
        build(a, 0, n, 0);
    }
};

SegmentTree Tree;

int main()
{
    cin.tie(0)->sync_with_stdio(0);
    int n, m, num;
    cin >> n >> m >> num;
    cout << n - 1 << ' ' << n << "\n";

}
