#include <iostream>
#include <vector>
#include <algorithm>
#include <string>
#include <bitset>
#include <unordered_set>

using namespace std;

using ll = long long;

const ll MOD = 1e9 + 7;

pair<ll, ll> solve_DAG(vector<vector<ll>> dag) {
	ll n = dag.size();
	ll cnt = 0;
	vector<pair<ll, ll>> edges;
	for (int i = 0; i < n; i++) {
		for (auto j : dag[i]) edges.push_back({ i, j });
	}
	for (auto e : edges) {
		vector<vector<ll>> dag2(n);
		for (int i = 0; i < n; i++) {
			for (auto p : dag[i]) {
				if (i == e.first && p == e.second) continue;
				dag2[p].push_back(i);
			}
		}
		vector<bool> can_get(n);
		can_get[e.first] = true;
		for (int i = e.first + 1; i <= e.second; i++) {
			for (auto p : dag2[i]) can_get[i] = can_get[i] || can_get[p];
		}
		if (can_get[e.second]) cnt++;
	}
	return { cnt, 1 };
}

void dfs(ll i, vector<vector<ll>>& graph, vector<bool>& used) {
	used[i] = true;
	for (auto p : graph[i]) {
		if (!used[p]) dfs(p, graph, used);
	}
}

pair<ll, ll> solve_component(vector<vector<ll>> graph) {
	ll cnt = 0;
	ll n = graph.size();
	vector<pair<ll, ll>> edges;
	for (int i = 0; i < graph.size(); i++) {
		for (auto j : graph[i]) {
			edges.push_back({ i, j });
		}
	}
	for (auto e : edges) {
		vector<vector<ll>> ngraph(n);
		for (int i = 0; i < n; i++) {
			for (auto j : graph[i]) {
				if (i == e.first && j == e.second) continue;
				ngraph[i].push_back(j);
			}
		}
		vector<bool> used(n, false);
		dfs(e.first, ngraph, used);
		if (used[e.second]) continue;
		used = vector<bool>(n, false);
		dfs(e.second, ngraph, used);
		if (!used[e.first]) continue;
		cnt++;
		graph[e.second].push_back(e.first);
	}
	return { cnt, 1 };
}

vector<ll> depth;
vector<vector<ll>> graph2;
vector<ll> color;
ll dfs(ll i, vector<vector<ll>>& graph) {
	color[i] = 1;
	ll upto = depth[i];
	for (auto p : graph[i]) {
		if (color[p] == 2) continue;
		if (color[p] == 1) {
			upto = min(upto, depth[p]);
			continue;
		}
		depth[p] = depth[i] + 1;
		ll u = dfs(p, graph);
		if (u <= depth[i]) {
			graph2[i].push_back(p);
			graph2[p].push_back(i);
		}
		upto = min(upto, u);
	}
	color[i] = 1;
	return upto;
}

void dfs(ll i, vector<vector<ll>>& graph, ll find, vector<ll>& used, ll ss) {
	used[i] = ss;
	for (auto p : graph[i]) {
		if (used[p] != ss) dfs(p, graph, find, used, ss);
		if (color[p] == find) color[i] = find;
	}
}

void dfs(ll i, ll c) {
	color[i] = c;
	for (auto p : graph2[i]) {
		if (color[p] != -1) continue;
		dfs(p, c);
	}
}

vector<ll> topsort;
void tdfs(int i, vector<vector<ll>>& graph, vector<bool>& used) {
	used[i] = true;
	for (auto p : graph[i]) {
		if (used[p]) continue;
		tdfs(p, graph, used);
	}
	topsort.push_back(i);
}

void paint(int i, vector<vector<ll>>& graph, vector<ll>& color, ll c) {
	color[i] = c;
	for (auto p : graph[i]) {
		if (color[p] == -1) paint(p, graph, color, c);
	}
}

vector<ll> divide_kss(vector<vector<ll>> graph) {
	ll n = graph.size();
	vector<bool> used(graph.size(), false);
	for (int i = 0; i < n; i++) {
		if (!used[i]) tdfs(i, graph, used);
	}
	reverse(topsort.begin(), topsort.end());
	vector<vector<ll>> regraph(n);
	for (int i = 0; i < n; i++) {
		for (auto j : graph[i]) {
			regraph[j].push_back(i);
		}
	}
	ll c = 0;
	vector<ll> color(n, -1);
	for (auto p : topsort) {
		if (color[p] != -1) continue;
		paint(p, regraph, color, c);
		c++;
	}
	return color;
}

/*
pair<ll, ll> solve_stupid(vector<vector<ll>> graph) {
	ll n = graph.size();
	vector<ll> masks;
	vector<pair<ll, ll>> edges;
	for (int i = 0; i < n; i++) {
		for (auto j : graph[i]) {
			if (j < i) continue;
			edges.push_back({ i, j });
		}
	}
	ll m = edges.size();
	for (ll i = 0; i < (1ll << m); i++) {

	}
}*/

int main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	ll n, m, g; cin >> n >> m >> g;
	vector<vector<ll>> graph(n);
	vector<pair<ll, ll>> edges;
	while (m--) {
		ll a, b; cin >> a >> b; a--; b--;
		graph[a].push_back(b);
		edges.push_back({ a, b });
	}
	vector<ll> dividing = divide_kss(graph);
	ll c = 0;
	for (auto p : dividing) c = max(c, p + 1);
	vector<vector<ll>> dag(c);
	unordered_set<ll> was;
	for (auto e : edges) {
		ll a = dividing[e.first], b = dividing[e.second];
		if (a == b) continue;
		if (a > b) swap(a, b);
		ll id = a * 1e9 + b;
		if (was.find(id) != was.end()) continue;
		was.insert(id);
		dag[a].push_back(b);
	}
	auto ans = solve_DAG(dag);
	vector<ll> cs(c);
	vector<ll> ingraph;
	for (auto p : dividing) {
		ingraph.push_back(cs[p]);
		cs[p]++;
	}
	vector<vector<vector<ll>>> graphs;
	for (auto p : cs) graphs.push_back(vector<vector<ll>>(p));
	for (auto p : edges) {
		ll c1 = dividing[p.first], c2 = dividing[p.second];
		ll a = ingraph[p.first], b = ingraph[p.second];
		graphs[c1][a].push_back(b);
	}
	for (auto g : graphs) {
		auto lans = solve_component(g);
		ans.first += lans.first;
		ans.second = (ans.second * lans.second) % MOD;
	}
	for (auto p : dividing) cout << p << " "; cout << endl;
	cout << ans.first << " " << ans.second << endl;
}