#ifdef ONPC
#define _GLIBCXX_DEBUG
#endif
#include <bits/stdc++.h>
#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/tree_policy.hpp>

using namespace std;
using namespace __gnu_pbds;

using ll = long long;
using ull = unsigned long long;
using ld = long double;
using i128 = __int128_t;

template<typename T>
using ordered_set = tree<T, null_type, less<T>, rb_tree_tag, tree_order_statistics_node_update>;


mt19937_64 rng(chrono::steady_clock().now().time_since_epoch().count());

const ll INF = 1e18;

vector<int> srt_a;

struct Line {
    ll k, b;
    Line() : k(0), b(-INF) {}
    Line(ll _k, ll _b) : k(_k), b(_b) {}

    ll get(int id) {
        return k * srt_a[id] + b;
    }
};

struct LiChao {
    vector<Line> lc;
    vector<vector<pair<Line, int>>> rb;

    void build(int n) {
        lc.resize(4 * n);
        rb.resize(4 * n);
    }

    ll get(int i, int x, int l, int r) {
        ll res = lc[x].get(i);
        if (l + 1 == r) return res;
        int m = (l + r) / 2;
        if (i < m) {
            res = max(res, get(i, 2 * x + 1, l, m));
        } else {
            res = max(res, get(i, 2 * x + 2, m, r));
        }
        return res;
    }

    void update(Line cur, int x, int l, int r) {
        int m = (l + r) / 2;
        rb[x].push_back({lc[x], -1});
        if (cur.get(m) > lc[x].get(m)) swap(lc[x].k, cur.k), swap(lc[x].b, cur.b);
        if (l + 1 == r) return;

        if (lc[x].get(l) < cur.get(l)) {
            rb[x].back().second = 2 * x + 1;
            update(cur, 2 * x + 1, l, m);
        }
        if (lc[x].get(r - 1) < cur.get(r - 1)) {
            rb[x].back().second = 2 * x + 2;
            update(cur, 2 * x + 2, m, r);
        }
    }

    void rollback() {
        int cur = 0;
        while (cur != -1) {
            auto [l, nxt] = rb[cur].back();
            rb[cur].pop_back();
            lc[cur] = l;
            cur = nxt;
        }
    }
};

int n;
LiChao lc1, lc2;
vector<Line> st1, st2;

void add(Line l) {
    st1.push_back(l);
    lc1.update(l, 0, 0, srt_a.size());
}

void del() {
    if (st2.empty()) {
        while (!st1.empty()) {
            st2.push_back(st1.back());
            lc2.update(st1.back(), 0, 0, srt_a.size());

            st1.pop_back();
            lc1.rollback();
        }
        assert(lc1.get(0, 0, 0, srt_a.size()) == -INF);
    }
    st2.pop_back();
    lc2.rollback();
}

ll get(int i) {
    return max(lc1.get(i, 0, 0, srt_a.size()), lc2.get(i, 0, 0, srt_a.size()));
}

ll solve(int m, ll T, vector<int> a, vector<int> b) {
    lc1.build(n);
    lc2.build(n);

    vector<int> ok(n + 1);
    for (int i = 1; i <= m; i++) ok[b[i]] = 1;

    vector<ll> pref(n + 1);
    for (int i = 1; i <= n; i++) pref[i] = a[i] + pref[i - 1];

    ll ans = 0;
    int r = n;
    for (int i = n; i >= 1; i--) {
        if (ok[i]) add({i, -pref[i]});
        while (pref[r] > pref[i] + T) {
            if (ok[r]) del();
            r--;
        }

        int id = lower_bound(srt_a.begin(), srt_a.end(), a[i]) - srt_a.begin();
        ll res = get(id);
        if (res == -INF) continue;

//        cout << i << ' ' << r << endl;
//        cout << "sz = " << st1.size() + st2.size() << '\n';
//        cout << "ans = " << res << ' ' << -i + (T + pref[i] + res) / a[i] << '\n';

        ans = max(ans, -i + (T + pref[i] + res) / a[i]);
    }

    while (st1.size() + st2.size()) del();

    return ans;
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int m;
    ll t;
    cin >> n >> m >> t;
    vector<int> a(n + 1), b(m + 1);
    for (int i = 1; i <= n; i++) cin >> a[i], srt_a.push_back(a[i]);
    for (int i = 1; i <= m; i++) cin >> b[i];

    sort(srt_a.begin(), srt_a.end());
    srt_a.resize(unique(srt_a.begin(), srt_a.end()) - srt_a.begin());

    ll ans = 0;
    ans = max(ans, solve(m, t, a, b));
    reverse(a.begin() + 1, a.end());
    reverse(b.begin() + 1, b.end());
    for (int i = 1; i <= m; i++) b[i] = n + 1 - b[i];
    ans = max(ans, solve(m, t, a, b));

    cout << ans << '\n';

    return 0;
}
