#include <iostream>
#include <random>
#include <numeric>
#include <algorithm>
#include <vector>
#include <map>
#include <set>

using namespace std;
using ll = long long;
using ull = unsigned long long;

constexpr ll maxn = 2e5 + 10, inf = 1e12;
mt19937 rng(463643);


ll n;
namespace DD {
    ll get_prior() {
        return static_cast<ll>(rng());
    }

    struct Node {
        ll sz = 0, val = 0, prior = 0;
        ll sum = 0;
        Node *l = nullptr, *r = nullptr;

        void init(ll _val);
    };

    constexpr ll max_nodes_cnt = 1e7;
    ll nodes_used = 0;
    Node tree[max_nodes_cnt];
    Node *zero_node = &tree[nodes_used++];
    Node *root = zero_node;

    void Node::init(ll _val) {
        prior = get_prior();
        sz = 1;
        sum = val = _val;
        l = r = zero_node;
    }

    Node *get_new_node(ll val) {
        Node *res = &tree[nodes_used++];
        res->init(val);
        return res;
    }

    ll size() {
        return root->sz;
    }

    void upd(Node *v) {
        if (v == zero_node) return;
        v->sz = v->l->sz + v->r->sz + 1;
        v->sum = v->l->sum + v->r->sum + v->val;
    }

    Node *merge(Node *a, Node *b) {
        if (a == zero_node) return b;
        if (b == zero_node) return a;
        if (a->prior > b->prior) {
            a->r = merge(a->r, b);
            upd(a);
            return a;
        }
        b->l = merge(a, b->l);
        upd(b);
        return b;
    }

    pair<Node *, Node *> split_sz(Node *v, ll sz) {
        if (v == zero_node) return {zero_node, zero_node};
        if (sz <= v->l->sz) {
            auto [l, vl] = split_sz(v->l, sz);
            v->l = vl;
            upd(v);
            return {l, v};
        }
        auto [vr, r] = split_sz(v->r, sz - v->l->sz - 1);
        v->r = vr;
        upd(v);
        return {v, r};
    }

    pair<Node *, Node *> split_val(Node *v, ll val) {
        if (v == zero_node) return {zero_node, zero_node};
        if (v->val >= val) {
            auto [l, vl] = split_val(v->l, val);
            v->l = vl;
            upd(v);
            return {l, v};
        }
        auto [vr, r] = split_val(v->r, val);
        v->r = vr;
        upd(v);
        return {v, r};
    }

    void insert(ll val) {
        auto [l1, r1] = split_val(root, val);
        auto m = get_new_node(val);
        root = merge(l1, merge(m, r1));
    }

    void erase(ll val) {
        auto [l1, r1] = split_val(root, val);
        auto [m, r2] = split_sz(r1, 1);
        root = merge(l1, r2);
    }

    pair<ll, ll> sum_cnt_less(ll val) {
        auto [l, r] = split_val(root, val);
        pair<ll, ll> ans{l->sum, l->sz};
        root = merge(l, r);
        return ans;
    }

    ll max_val() {
        auto [l, r] = split_sz(root, n - 1);
        ll ans = r->sum;
        root = merge(l, r);
        return ans;
    }

    ll all_sum() {
        return root->sum;
    }

    ll get_by_pos(ll pos) {
        auto [l1, r1] = split_sz(root, pos);
        auto [l2, r2] = split_sz(r1, 1);
        ll ans = l2->val;
        root = merge(l1, merge(l2, r2));
        return ans;
    }
}
ll tarr[maxn];
//ll pf[maxn];

ll sum_if_max_d(ll d) {
//    ll pos = lower_bound(arr, arr + n, d) - arr;
//    return pf[pos] + 1ll * d * (n - pos);
    auto [cs, cp] = DD::sum_cnt_less(d);
    return cs + 1ll * d * (n - cp);
}

struct Pans {
    ll eqp1, eqp2, rv;
};

Pans get_eq_poss(ll m, ll k) {
    ll l = -inf, r = DD::max_val();
    while (r - l != 1) {
        ll mid = (l + r) >> 1;
        if (sum_if_max_d(mid) >= DD::all_sum() - 1ll * m * k) r = mid;
        else l = mid;
    }
    ll eq_pos_1 = DD::sum_cnt_less(r).second;
//    ll eq_pos_1 = lower_bound(arr, arr + n, r) - arr;
    ll eq_pos_2 = n - (sum_if_max_d(r) - (DD::all_sum() - 1ll * m * k));
    return {eq_pos_1, eq_pos_2, r};
}

ll get_val(ll i, Pans pps) {
    if (i < pps.eqp1) return DD::get_by_pos(i);
    if (i - pps.eqp1 < n - pps.eqp2) return pps.rv - 1;
    return pps.rv;
}

void solve() {
    ll m0, k0, q;
    cin >> n >> m0 >> k0 >> q;
    for (ll i = 0; i < n; ++i) cin >> tarr[i];
    for (int i = 0; i < n; ++i) DD::insert(tarr[i]);
//    sort(arr, arr + n);
//    for (ll i = 0; i < n; ++i) pf[i + 1] = pf[i] + arr[i];
    auto f0 = get_eq_poss(m0, k0);
    for (ll i = 0; i < n; ++i) cout << get_val(i, f0) << " ";
    for (int _ = 0, t; _ < q; ++_) {
        cin >> t;
        if (t == 1) {
            int m, k, x;
            cin >> m >> k >> x;
            cout << get_val(x - 1, get_eq_poss(m, k)) << "\n";
        }
    }
}

int main() {
    ios::sync_with_stdio(false), cin.tie(nullptr);
    int t = 1;
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
    cin >> t;
#else
#ifdef TEST
    freopen("input.txt", "r", stdin);
#else
#endif
#endif
    while (t--) {
        solve();
        cout << "\n";
    }
    return 0;
}

//    for (int i = 0; i < n; ++i) {
//        for (int j = 0; j < n; ++j) dists[j] = 0;
//        dfs(i, 0);
//        dfs2(i);
//    }
//    int cnt = 0;
//    for (int i = 0; i < m; ++i) cnt += need[i];
//    cout << 0 << "\n";