#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <map>
#include <unordered_set>
#include <tuple>
#include <random>
#include <queue>
#include <cmath>

using namespace std;
using ll = long long;

constexpr int INF = 1e9 + 7;
constexpr ll LINF = 1e18 + 7;

mt19937 rnd(37);

struct node {
    ll x = 0;
    ll y = -1;
    ll s = 0;
    ll cnt = 0;
    ll push = 0;
    node* l = nullptr, * r = nullptr;
    node(ll x_) {
        x = x_;
        s = x_;
        y = rnd();
        cnt = 1;
    }
    node() {
        x = 0;
        y = rnd();
    }
};

int get_val(node* root) {
    if (root == nullptr) return -1;
    return root->x;
}

int get_cnt(node* root) {
    if (root == nullptr) return 0;
    return root->cnt;
}

ll get_sum(node* root) {
    if (root == nullptr) return 0;
    return root->s;
}

void update_node(node* root) {
    if (root == nullptr) return;
    root->cnt = get_cnt(root->l) + get_cnt(root->r) + 1;
    root->s = get_sum(root->l) + get_sum(root->r) + root->x;
}

void push(node* root) {
    if (!root) return;
    if (root->l) {
        root->l->x += root->push;
        root->l->s += root->push * root->l->cnt;
        root->l->push += root->push;
    }
    if (root->r) {
        root->r->x += root->push;
        root->r->s += root->push * root->r->cnt;
        root->r->push += root->push;
    }
    root->push = 0;
}

pair<node*, node*>split(node* root, int k) {
    if (root == nullptr) return { nullptr, nullptr };
    push(root);
    if (root->l->cnt >= k) {
        auto [lft, rgt] = split(root->l, k);
        root->l = rgt;
        update_node(root);
        return { lft, root };
    }
    auto [lft, rgt] = split(root->r, k - root->l->cnt - 1);
    root->r = lft;
    update_node(root);
    return { root, rgt };
}

node* merge(node* a, node* b) {
    if (a == nullptr) return b;
    if (b == nullptr) return a;
    if (a->y > b->y) {
        a->r = merge(a->r, b);
        update_node(a);
        return a;
    }
    b->l = merge(a, b->l);
    update_node(b);
    return b;
}

void add(node* root, ll x) {
    if (!root) return;
    root->x += x;
    root->s += root->cnt * x;
    root->push += x;
}

void print(node* root) {
    if (!root) return;
    push(root);
    print(root->l);
    cout << root->x << ' ';
    print(root->r);
}

int main()
{
    cin.tie(0)->sync_with_stdio(0);
    int n, m0, k0, q;
    cin >> n >> m0 >> k0 >> q;
    vector<int>a(n);
    for (int i = 0; i < n; ++i) cin >> a[i];
    sort(a.begin(), a.end());
    /*node* root;
    for (int i = 0; i < n; ++i) {
        root = merge(root, new node(a[i]));
    }
    int l = 0, r = n;
    while (r - l > 1) {
        int mid = (l + r) / 2;
        auto [lft, rgt] = split(root, mid);
        ll s1 = get_val(lft) * get_cnt(rgt);
        ll s2 = get_sum(rgt);
        if (s2 - s1 >= m0) l = mid;
        else r = mid;
    }
    print(root); cout << "\n";*/
    for (int i = 0; i < m0; ++i) {
        for (int j = n - 1; j >= n - k0; --j) {
            a[j]--;
        }
        if (a[n - k0] < a[n - k0 - 1]) sort(a.begin(), a.end());
    }
    for (auto x : a) cout << x << ' ';
    cout << "\n";
}
