#include <iostream>
#include <iomanip>
#include <vector>
#include <set>
#include <map>
#include <algorithm>
#include <cmath>
#include <bitset>
#include <unordered_map>
#include <random>

#define all(v) v.begin(), v.end()
#define len(v) (int)v.size()
#define coutd cout << fixed << setprecision(20)
#pragma GCC optimize("Ofast,unroll-loops")

using namespace std;
using ll = long long;
using ld = long double;

const ll IINF = 2e9;

struct Node
{
    int mn, mx, sc;
};

Node operator+(Node a, Node b)
{
    return {min(a.mn, b.mn), max(a.mx, b.mx), 0};
}

struct Seg_tree
{
    vector<Node> tree;
    vector<int> a;
    int n;
    Seg_tree(vector<int> &b)
    {
        a = b;
        n = len(a);
        tree.resize(4 * n);
        build(0, 0, n);
    }
    void push(int v, int l, int r)
    {
        int m = (l + r) / 2;
        tree[2 * v + 1].mn += tree[v].sc;
        tree[2 * v + 2].mn += tree[v].sc;
        tree[2 * v + 1].mx += tree[v].sc;
        tree[2 * v + 2].mx += tree[v].sc;
        tree[2 * v + 1].sc += tree[v].sc;
        tree[2 * v + 2].sc += tree[v].sc;
        tree[v].sc = 0;
    }
    void build(int v, int l, int r)
    {
        if (r - l == 1)
        {
            tree[v] = {a[l], a[l], 0};
            return;
        }
        int m = (l + r) / 2;
        build(2 * v + 1, l, m);
        build(2 * v + 2, m, r);
        tree[v] = tree[2 * v + 1] + tree[2 * v + 2];
    }
    void update(int v, int l, int r, int ql, int qr, int val)
    {
        if (l >= qr || r <= ql) return;
        if (l >= ql && r <= qr)
        {
            tree[v].mn += val;
            tree[v].mx += val;
            tree[v].sc += val;
            return;
        }
        push(v, l, r);
        int m = (l + r) / 2;
        update(2 * v + 1, l, m, ql, qr, val);
        update(2 * v + 2, m, r, ql, qr, val);
        tree[v] = tree[2 * v + 1] + tree[2 * v + 2];
    }
    ll get_sum(int v, int l, int r, int ql, int qr)
    {
        if (l >= qr || r <= ql) return 0;
        if (l >= ql && r <= qr) return tree[v].mn;
        push(v, l, r);
        int m = (l + r) / 2;
        return get_sum(2 * v + 1, l, m, ql, qr) + get_sum(2 * v + 2, m, r, ql, qr);
    }
    int findleft(int v, int l, int r, int k)
    {
        if (r - l == 1)
        {
            return l;
        }
        int m = (l + r) / 2;
        if (tree[2 * v + 1].mx >= k)
        {
            return findleft(2 * v + 1, l, m, k);
        }
        return findleft(2 * v + 2, m, r, k);
    }
    int findright(int v, int l, int r, int k)
    {
        if (r - l == 1)
        {
            return l;
        }
        int m = (l + r) / 2;
        if (tree[2 * v + 2].mn <= k)
        {
            return findright(2 * v + 2, m, r, k);
        }
        return findright(2 * v + 1, l, m, k);
    }
};

int main()
{
    cin.tie(0)->sync_with_stdio(0);
    int n, m, k, q;
    cin >> n >> m >> k >> q;
    vector<int> a(n);
    for (int i = 0; i < n; ++i)
    {
        cin >> a[i];
    }
    sort(all(a));
    Seg_tree tree(a);
    for (int i = 0; i < m; ++i)
    {
        int val = tree.get_sum(0, 0, n, n - k, n - k + 1);
        int r1 =tree.findleft(0, 0, n, val);
        int l2 =tree.findright(0, 0, n, val);
        tree.update(0, 0, n, l2 + 1, n, -1);
        int need = (l2 - (n - k) + 1);
        tree.update(0, 0, n, r1, r1 + need, -1);
    }
    for (int i = 0; i < n; ++i)
    {
        cout << tree.get_sum(0, 0, n, i, i + 1) << ' ';
    }

    return 0;
}
