#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

struct Node {
    ll cnt, sum;
};

Node merge(Node &a, Node &b) {
    Node res{};
    res.cnt = a.cnt + b.cnt;
    res.sum = a.sum + b.sum;
    return res;
}

struct SegmentTree {
    vector<Node> tree;
    vector<ll> arr;
    ll n;

    SegmentTree() = default;

    explicit SegmentTree(vector<ll> &a) : arr(a) {
        n = ll(a.size());
        tree.assign(4 * n, {0, 0});
    }

    void update(ll v, ll l, ll r, ll ind, ll val) {
        if (ind < l || r <= ind) {
            return;
        }
        if (l == r - 1) {
            tree[v].cnt += val;
            tree[v].sum += val * arr[l];
            return;
        }
        ll m = (l + r) / 2;
        update(2 * v + 1, l, m, ind, val);
        update(2 * v + 2, m, r, ind, val);
        tree[v] = merge(tree[2 * v + 1], tree[2 * v + 2]);
    }

    void update(ll ind, ll val) {
        update(0, 0, n, ind, val);
    }

    Node get(ll v, ll l, ll r, ll ql, ll qr) {
        if (qr <= l || r <= ql) {
            return {0, 0};
        }
        if (ql <= l && r <= qr) {
            return tree[v];
        }
        ll m = (l + r) / 2;
        Node x = get(2 * v + 1, l, m, ql, qr);
        Node y = get(2 * v + 2, m, r, ql, qr);
        return merge(x, y);
    }

    Node get(ll ql, ll qr) {
        return get(0, 0, n, ql, qr);
    }

    ll AYE(ll v, ll l, ll r, ll k) {
        if (l == r - 1) {
            return arr[l];
        }
        ll m = (l + r) / 2;
        if (tree[2 * v + 1].cnt >= k) {
            return AYE(2 * v + 1, l, m, k);
        } else {
            return AYE(2 * v + 2, m, r, k - tree[2 * v + 1].cnt);
        }
    }

    ll AYE(ll k) {
        return AYE(0, 0, n, k);
    }
};

ll n, m0, k0, q;
vector<ll> a;
vector<ll> a1;
SegmentTree tree;
map<ll, ll> mp;
ll N;

ll solve(ll m, ll k, ll x) {
    ll l = -1, r = N;
    while (r - l > 1) {
        ll mid = (l + r) / 2;
        Node res = tree.get(mid, N);
        ll sum = res.sum - res.cnt * a1[mid];
        if (sum >= m) {
            l = mid;
        } else {
            r = mid;
        }
    }
    if (l == -1) {
        Node res = tree.get(0, N);
        ll a0 = tree.AYE(1);
        m -= res.sum - res.cnt * a0;
        ll gg = m / n;
        if (x < m % n) {
            return a0 - gg - 1;
        } else {
            return a0 - gg;
        }
    }
    if (x <= l) {
        return tree.AYE(x + 1);
    }
    l++;
    Node res = tree.get(l, N);
    ll al = tree.AYE(l + 1);
    m -= res.sum - res.cnt * al;
    ll gg = m / (n - l);
    if (x < l + m % (n - l)) {
        return al - gg - 1;
    } else {
        return al - gg;
    }
}

int main() {
#ifdef DEBUG_Q
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);

    cin >> n >> m0 >> k0 >> q;
    a.assign(n, 0);
    for (ll i = 0; i < n; i++) {
        cin >> a[i];
    }
    for (ll i = 0; i < n; i++) {
        a1.push_back(a[i]);
    }
    vector<vector<ll>> queries(q);
    for (ll i = 0; i < q; i++) {
        ll t;
        cin >> t;
        if (t == 1) {
            ll m, k, x;
            cin >> m >> k >> x;
            queries[i] = {t, m, k, x};
        } else if (t == 2) {
            ll x, v;
            cin >> x >> v;
            queries[i] = {t, x, v};
            a1.push_back(v);
        } else {
            ll m, k, l, r;
            cin >> m >> k >> l >> r;
        }
    }
    sort(a1.begin(), a1.end());
    a1.resize(unique(a1.begin(), a1.end()) - a1.begin());
    N = ll(a1.size());
    for (ll i = 0; i < N; i++) {
        mp[a1[i]] = i;
    }
    tree = SegmentTree(a1);
    for (ll i = 0; i < n; i++) {
        tree.update(mp[a[i]], 1);
    }
    for (ll i = 0; i < n; i++) {
        cout << solve(m0, k0, i) << ' ';
    }
    cout << '\n';
    for (auto &i: queries) {
        if (i[0] == 1) {
            ll m = i[1], k = i[2], x = i[3];
            cout << solve(m, k, x - 1) << '\n';
        } else {
            ll x = i[1], v = i[2];
            x--;
            tree.update(mp[a[x]], -1);
            a[x] = v;
            tree.update(mp[a[x]], 1);
        }
    }
}